/**
 * A React component to view a PDF document
 *
 * @see https://react-pdf-viewer.dev
 * @license https://react-pdf-viewer.dev/license
 * @copyright 2019-2022 Nguyen Huu Phuoc <me@phuoc.ng>
 */

import * as React from 'react';
import { createStore } from '@react-pdf-viewer/core';
import type { Plugin, PluginFunctions, PluginOnDocumentLoad, ViewerState } from '@react-pdf-viewer/core';

import { Cover } from './Cover';
import { defaultSpinner, SpinnerContext } from './SpinnerContext';
import { ThumbnailListWithStore } from './ThumbnailListWithStore';
import type { CoverProps } from './types/CoverProps';
import type { RenderCurrentPageLabel } from './types/RenderCurrentPageLabelProps';
import type { RenderThumbnailItem } from './types/RenderThumbnailItemProps';
import type { StoreProps } from './types/StoreProps';

export interface ThumbnailsProps {
    renderThumbnailItem?: RenderThumbnailItem;
}

export interface ThumbnailPlugin extends Plugin {
    Cover: (props: CoverProps) => React.ReactElement;
    Thumbnails(props?: ThumbnailsProps): React.ReactElement;
}

export interface ThumbnailPluginProps {
    renderCurrentPageLabel?: RenderCurrentPageLabel;
    // The spinner that replaces the default `Spinner` component
    // For example, it is displayed when loading the cover or thumbnail of a page
    renderSpinner?: () => React.ReactElement;
}

export const thumbnailPlugin = (pluginProps?: ThumbnailPluginProps): ThumbnailPlugin => {
    const store = React.useMemo(() => createStore<StoreProps>({}), []);
    const [docId, setDocId] = React.useState('');

    const CoverDecorator = (props: CoverProps) => (
        <Cover {...props} renderSpinner={pluginProps?.renderSpinner} store={store} />
    );

    const ThumbnailsDecorator = React.useCallback(
        (props?: ThumbnailsProps) => (
            <SpinnerContext.Provider value={{ renderSpinner: pluginProps?.renderSpinner || defaultSpinner }}>
                <ThumbnailListWithStore
                    renderCurrentPageLabel={pluginProps?.renderCurrentPageLabel}
                    renderThumbnailItem={props?.renderThumbnailItem}
                    store={store}
                />
            </SpinnerContext.Provider>
        ),
        [docId]
    );

    return {
        install: (pluginFunctions: PluginFunctions) => {
            store.update('jumpToPage', pluginFunctions.jumpToPage);
        },
        onDocumentLoad: (props: PluginOnDocumentLoad) => {
            setDocId(props.doc.loadingTask.docId);
            store.update('doc', props.doc);
        },
        onViewerStateChange: (viewerState: ViewerState) => {
            store.update('currentPage', viewerState.pageIndex);
            store.update('pageHeight', viewerState.pageHeight);
            store.update('pageWidth', viewerState.pageWidth);
            return viewerState;
        },
        Cover: CoverDecorator,
        Thumbnails: ThumbnailsDecorator,
    };
};
